package db.netprobe.api.page;

import db.netprobe.api.DBVal;
import db.netprobe.api.Point;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 功能描述
 *
 * @author: scott
 * @date: 2022年09月27日 10:01 AM
 */
public class MultiPointRangePagingQuery implements PagingQuery{
    int pageSize;
    long start;
    long end;
    int maxPageCount = 0;
    Map<Point, RangePagingQuery> rangePagingQueryMap;

    public MultiPointRangePagingQuery(long start, long end, int pageSize,
                                      Map<Point, RangePagingQuery> rangePagingQueryMap) {
        this.pageSize = pageSize;
        this.start = start;
        this.end = end;
        this.rangePagingQueryMap = rangePagingQueryMap;

        for (RangePagingQuery query : this.rangePagingQueryMap.values()) {
            if(query.getPageCount()>maxPageCount)
                maxPageCount = query.getPageCount();
        }
    }

    public int getPageSize() {
        return pageSize;
    }

    public Map<Point, List<DBVal>> getPage(int pageIndex) throws Exception {
//        if(this.maxPageCount==0&&pageIndex==0) {
//            return new HashMap<>();
//        }

        if(pageIndex>=maxPageCount || pageIndex<0) throw new Exception("pageIndex>=maxPageCount || pageIndex<0");

        Map<Point, List<DBVal>> result = new HashMap<>();
        for(Map.Entry<Point, RangePagingQuery> entry : this.rangePagingQueryMap.entrySet()) {
            List<DBVal> dbValList = new ArrayList<>();
            try {
                dbValList = entry.getValue().getPage(pageIndex);
            } catch (Exception e) {

            }
            result.put(entry.getKey(), dbValList);
        }
        return result;
    }

    public List<DBVal> getPage(Point point, int pageIndex) throws Exception {
        RangePagingQuery rangePagingQuery = this.rangePagingQueryMap.get(point);
        if(rangePagingQuery==null) throw new Exception("point error");
        return rangePagingQuery.getPage(pageIndex);
    }

    public int getMaxPageCount() {
        return maxPageCount;
    }
}
