package db.netprobe.entity;

import db.netprobe.Globals;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.text.ParseException;

@Data
@NoArgsConstructor
public abstract class BasePackage {
    private int magic;
    private int version;

    private short operationType;
    private short responseCode;
    private int packageId;
    private short operationCode;
    private short packageCode;
    private short packageLength;

    public BasePackage(int version, short operationType, short responseCode, int packageId, short operationCode, short packageCode, short packageLength) {
        this.magic = Globals.MAGIC;
        this.version = version;
        this.operationType = operationType;
        this.responseCode = responseCode;
        this.packageId = packageId;
        this.operationCode = operationCode;
        this.packageCode = packageCode;
        this.packageLength = packageLength;
    }

    public void setHeader(ByteBuf buf) {
        magic = buf.readInt(); // skip magic
        version = buf.readInt();
        operationType = buf.readShort();
        responseCode = buf.readShort();
        operationCode = buf.readShort();
        packageCode = buf.readShort();
        packageLength = buf.readShort();
    }

    public abstract void setPayload(ByteBuf buf) throws ParseException;

    public ByteBuf getHeader() {
        ByteBuf buf = Unpooled.buffer(18);
        buf.writeInt(magic);
        buf.writeInt(version);
        buf.writeShort(operationType);
        buf.writeShort(responseCode);
        buf.writeShort(operationCode);
        buf.writeShort(packageCode);
        buf.writeShort(packageLength);

        return buf;
    }

    public abstract ByteBuf getPayload();

}
