package db.netprobe.entity;

import db.netprobe.Globals;
import db.netprobe.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

@Data
@NoArgsConstructor
@Slf4j
public class DataPackage extends BasePackage {
    private short probeId;
    private byte port;

    private byte dataStrategy;
    private byte reserve;
    private byte clockSync;
    private short year;
    private short day;
    private short hour;
    private short minute;
    private short second;
    private int microSecond;
    private long timestamp;

    private String sMac;
    private String dMac;
    private String sIp;
    private String dIp;
    private String sPort;
    private String dPort;

    private byte[] data;

    public DataPackage(int version, short operationType, short responseCode, int packageId, short operationCode,
                       short packageCode, short probeId, byte port, byte dataStrategy,
                       byte reserve, byte clockSync, short year, short day, short hour, short minute, short second,
                       int microSecond, byte[] data) {
        super(version, operationType, responseCode, packageId, operationCode, packageCode, (short) (data.length + 20));
        this.probeId = probeId;
        this.port = port;
        this.dataStrategy = dataStrategy;
        this.reserve = reserve;
        this.clockSync = clockSync;
        this.year = year;
        this.day = day;
        this.hour = hour;
        this.minute = minute;
        this.second = second;
        this.microSecond = microSecond;
        this.data = data;
        data2Meta();
    }

    @Override
    public String toString() {
        return "DataPackage{" +
                "probeId=" + probeId +
                ", port=" + port +
                ", dataStrategy=" + dataStrategy +
                ", reserve=" + reserve +
                ", clockSync=" + clockSync +
                ", year=" + year +
                ", day=" + day +
                ", hour=" + hour +
                ", minute=" + minute +
                ", second=" + second +
                ", microSecond=" + microSecond +
                ", sMac=" + sMac +
                ", dMac=" + dMac +
                ", sIp=" + sIp +
                ", dIp=" + dIp +
                ", sPort=" + sPort +
                ", dPort=" + dPort +
                ", timestamp=" + timestamp +
                ", data=" + ConvertUtils.bytes2HexArray(data) +
                '}';
    }


    @Override
    public void setPayload(ByteBuf buf) throws ParseException {
        probeId = buf.readShort();
        port = buf.readByte();
        dataStrategy = buf.readByte();
        reserve = buf.readByte();
        clockSync = buf.readByte();
        year = buf.readShort();
        day = buf.readShort();
        hour = buf.readShort();
        minute = buf.readShort();
        second = buf.readShort();
        microSecond = buf.readInt();
        data = new byte[buf.readableBytes()];
        buf.readBytes(data);

        data2Meta();
    }

    @Override
    public ByteBuf getPayload() {
        ByteBuf buf = Unpooled.buffer(18 + data.length);
        buf.writeShort(probeId);
        buf.writeByte(port);
        buf.writeByte(dataStrategy);
        buf.writeByte(reserve);
        buf.writeByte(clockSync);
        buf.writeShort(year);
        buf.writeShort(day);
        buf.writeShort(hour);
        buf.writeShort(minute);
        buf.writeShort(second);
        buf.writeInt(microSecond);
        buf.writeBytes(data);
        return buf;
    }

    private long phraseTimeStamp() throws ParseException {
        String time = String.format("%d-%d %d:%d:%d", year, day, hour, minute, second);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-DD HH:mm:ss");
        Date date = simpleDateFormat.parse(time);
        return date.getTime() * 1000 + microSecond;
    }


    private void data2Meta() {
        byte[] sMacByte = Arrays.copyOfRange(data, 6, 12);
        byte[] dMacByte = Arrays.copyOfRange(data, 0, 6);
        byte[] sIpByte = Arrays.copyOfRange(data, 26, 30);
        byte[] dIpByte = Arrays.copyOfRange(data, 30, 34);
        byte[] sPortByte = Arrays.copyOfRange(data, 34, 36);
        byte[] dPortByte = Arrays.copyOfRange(data, 36, 38);

        sMac = String.format("%12s", Long.toHexString(ConvertUtils.macBytes2Long(sMacByte))).replace(" ", "0");
        dMac = String.format("%12s", Long.toHexString(ConvertUtils.macBytes2Long(dMacByte))).replace(" ", "0");
        sIp = ConvertUtils.ip2String(ConvertUtils.bytes2Int(sIpByte));
        dIp = ConvertUtils.ip2String(ConvertUtils.bytes2Int(dIpByte));
        sPort = String.valueOf(ConvertUtils.bytes2Short(sPortByte));
        dPort = String.valueOf(ConvertUtils.bytes2Short(dPortByte));
        try {
            timestamp = phraseTimeStamp();
        } catch (ParseException e) {
            log.warn("转换时间失败：{}",this);
        }
    }

}
