package db.netprobe.entity;

import db.netprobe.Globals;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.text.ParseException;
import java.util.Arrays;

@Data
public class LogFileHeader {
    private int[] indices;
    private long minTime;
    private long maxTime;
    private int bodyLength;
    private int headerExtLength;
    private int nextWriteIndex = -1;

    public LogFileHeader() {
        this.indices = new int[1024 * 64];
        Arrays.fill(this.indices, -1);
        this.minTime = 0;
        this.maxTime = 0;
        this.bodyLength = 0;
        this.headerExtLength = 0;
    }

    public byte[] getPayload() {
        ByteBuf buf = Unpooled.buffer(indices.length * 4 + 24 + headerExtLength);
        for (int i = 0; i < indices.length; i++) {
            buf.writeInt(indices[i]);
        }
        buf.writeLong(minTime);
        buf.writeLong(maxTime);
        buf.writeInt(bodyLength);
        buf.writeInt(headerExtLength);
        return buf.array();
    }

    public void setPayload(byte[] buffer) {

        ByteBuf buf = Unpooled.buffer(buffer.length);
        buf.writeBytes(buffer);

        for (int i = 0; i < 1024 * 64; i++) {
            indices[i] = buf.readInt();
            if(indices[i] == -1 && nextWriteIndex < 0 ){
                nextWriteIndex = i;
            }
        }

        minTime = buf.readLong();
        maxTime = buf.readLong();
        bodyLength = buf.readInt();
        headerExtLength = buf.readInt();
    }
}
