package db.netprobe.handler;

import db.netprobe.api.DBApiEntry;
import db.netprobe.api.Point;
import db.netprobe.api.PointVals;
import db.netprobe.entity.DataPackage;
import db.netprobe.service.LogFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.text.ParseException;
import java.util.HashMap;

@Component
@Slf4j
public class DataPackageHandler {

    @Value("${reso-db.host}")
    private String host;

    @Value("${reso-db.port}")
    private int port;

    private DBApiEntry entry;

    @Autowired
    private LogFileService packetFileMetaService;

    public DataPackageHandler() {
        this.entry = DBApiEntry.initApiEntry(host, port);
    }

    public void handlerDataPackage(DataPackage dataPackage) throws ParseException, IOException {
        log.info("开始保存数据包{}", dataPackage);

        log.info("开始保存至磁盘");
        int index = packetFileMetaService.appendLog(dataPackage);
        log.info("写入磁盘成功磁盘");

        // 写入数据库
        log.info("开始尝试插入resoDB");

        String metricName = "netProbe";
        HashMap<String, String> tags = new HashMap<>();
        tags.put("sMac", dataPackage.getSMac());
        tags.put("dMac", dataPackage.getDMac());
        tags.put("sPort", dataPackage.getSPort());
        tags.put("dPort", dataPackage.getDPort());
        tags.put("sIp", dataPackage.getSIp());
        tags.put("dIp", dataPackage.getDIp());
        tags.put("probeId", String.valueOf(dataPackage.getProbeId()));
        Point point = new Point(metricName, tags);

        long timeStamp = System.currentTimeMillis();
        long[] utcTimes = new long[1];
        double[] values = new double[1];

        utcTimes[0] = timeStamp;
        values[0] = index;
        PointVals pointVals = new PointVals(point, 1, utcTimes, values);
        int insertCount = entry.insertSinglPoint(pointVals);
        log.info("插入resoDB结束，插入{}条", insertCount);
    }


}
