package db.netprobe.netty;

import db.netprobe.Globals;
import db.netprobe.entity.BasePackage;
import db.netprobe.entity.DataPackage;
import db.netprobe.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import io.netty.handler.codec.MessageToMessageDecoder;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.List;

@Slf4j
public class Decoder extends MessageToMessageDecoder<DatagramPacket> {

    @Override
    protected void decode(ChannelHandlerContext ctx, DatagramPacket datagramPacket, List<Object> out) throws Exception {
        ByteBuf in = datagramPacket.content();
        byte[] bytesReady = new byte[in.readableBytes()];
        in.getBytes(0, bytesReady);
        log.info("解码器收到数据：{} 从 {}", ConvertUtils.bytes2HexArray(bytesReady), ctx);

        int magic = in.getInt(0);

        if (magic != Globals.MAGIC) {
            log.warn("魔数错误，消息为：{}", ConvertUtils.bytes2HexArray(bytesReady));
            return;
        }

        int ver = in.getInt(4);
        short operationType = in.getShort(8);
        short responseCode = in.getShort(10);
        int packageId = in.getInt(12);
        short operationCode = in.getShort(12);
        short packageCode = in.getShort(14);
        short packageLength = in.getShort(16);

        BasePackage basePackage;


        if(operationType == 0x0003 && operationCode == 0x0006)
        {
            basePackage = new DataPackage();
        } else {
            return;
        }

        // 把byteBuf中的内容填充到包对象中
        basePackage.setHeader(in);

        byte[] payload = Arrays.copyOfRange(bytesReady, 18, bytesReady.length);

        basePackage.setPayload(in);

        out.add(basePackage);
    }
}


