package db.netprobe.netty;

import db.netprobe.entity.BasePackage;
import db.netprobe.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToByteEncoder;
import io.netty.handler.codec.MessageToMessageEncoder;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;

import java.net.InetSocketAddress;
import java.util.List;


// 在这里编码包
@Slf4j
public class Encoder extends MessageToMessageEncoder<BasePackage> {

   private String host;
   private Integer port;
   InetSocketAddress remoteAddress;

   public Encoder(String host, Integer port) {
      this.host = host;
      this.port = port;
      this.remoteAddress = new InetSocketAddress(host, port);
   }

   @Override
   protected void encode(ChannelHandlerContext channelHandlerContext, BasePackage basePackage, List<Object> out) throws Exception {
      //写入并传送数据

      ByteBuf header = basePackage.getHeader();
      ByteBuf payload = basePackage.getPayload();

      ByteBuf buf = channelHandlerContext.alloc().buffer(header.readableBytes()+ payload.readableBytes());

      buf.writeBytes(header);
      buf.writeBytes(payload);

      DatagramPacket packet = new DatagramPacket(buf, remoteAddress);
      out.add(packet);



      byte[] bytesReady = new byte[buf.readableBytes()];
      buf.getBytes(0, bytesReady);
      log.info("编码器发送数据：{} 到 {}", ConvertUtils.bytes2HexArray(bytesReady), channelHandlerContext);
   }
}