package db.netprobe.netty;

import db.netprobe.entity.DataPackage;
import db.netprobe.util.ContentUtils;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.*;

import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.PostConstruct;

@Slf4j
public class NettyClient {

    private EventLoopGroup group = new NioEventLoopGroup();

    private Channel ch;

    @PostConstruct
    public void start() throws InterruptedException {
        log.info("开始创建client");

        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group)
                .channel(NioDatagramChannel.class)
                .option(ChannelOption.SO_BROADCAST, true)
                .handler(new ChannelInitializer<NioDatagramChannel>() {
                    @Override
                    public void initChannel(final NioDatagramChannel ch) throws Exception {

                        ChannelPipeline p = ch.pipeline();
                        p.addLast(new Encoder("127.0.0.1", 1208));
                        p.addLast(new NettyClientHandler());
                    }
                });

        ch = bootstrap.bind(1200).sync().channel();
        log.info("创建client成功");

    }

    public void send(Object content) {
//        ch.writeAndFlush(new DatagramPacket(Unpooled.copiedBuffer("123123", CharsetUtil.UTF_8),
//                new InetSocketAddress("127.0.0.1", 1208)));


        ch.writeAndFlush(content);
    }

    public static void main(String[] args) throws InterruptedException {
        NettyClient client = new NettyClient();
        client.start();
        log.debug("发送");


        int version = 0x21030001;
        short operationType = 0x3;
        short responseCode = 200;
        short operationCode = 0x0006;
        short packageCode = 0;
        short packageLength = 0x0054;
        short probeId = 0x0010;
        byte port = 0x01;
        byte dataStrategy = 0;
        byte reserve = 0;
        byte clockSync = (byte) 0xAA;
        short year = 2014;
        short day = 3;
        short hour = 8;
        short minute = 0x2e;
        short second = 0x10;
        int microSecond = 0x000f0b99;
//        byte[] data = {0x00, 0x00, 0x02, 0x55, 0x55, 0x55, 0x00, 0x00, 0x02, 0x44, 0x44, 0x66, 0x08, 0x00, 0x45, 0x00,
//                0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11, 0x3E, (byte) 0x87, 0x03, 0x1B, 0x1B, 0x02, 0x03, 0x1B,
//                0x1B, 0x01, 0x04, 0x00, 0x04, 0x01, 0x00, 0x1A, 0x00, 0x00, (byte) 0xA5, 0x00, 0x66, (byte) 0x80, 0x00, 0x00,
//                0x00, 0x3c, 0x00, 0x01, (byte) 0x99, 0x42, 0x59, (byte) 0xAF, 0x03, (byte) 0xDB, 0x00, 0x23, (byte) 0xc0, (byte) 0x96, 0x0B, 0x78};

        byte[] data = ContentUtils.generatorRandomData(0x00000002555556L, 0x000002444455L, "192.168.1.1",
                "192.168.31.1", (short) 8080, (short) 9090);

                DataPackage dataPackage = new DataPackage(version, operationType, responseCode, 0, operationCode,
                packageCode, probeId, port, dataStrategy, reserve, clockSync, year, day, hour, minute, second,
                microSecond, data);
        System.out.println("dataPackage = " + dataPackage);
        client.send(dataPackage);
    }


}
