package db.netprobe.service;

import db.netprobe.entity.DataPackage;
import db.netprobe.entity.LogFileHeader;
import db.netprobe.entity.LogItem;
import db.netprobe.util.ContentUtils;
import db.netprobe.util.FileUtils;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.ParseException;

@Service
@Data
@Slf4j
public class LogFileService {
    private String fileName = "1.bin"; // 当前应写入的文件名

    public int appendLog(DataPackage dataPackage) throws IOException {
        // 封装数据
        LogItem logItem = new LogItem((short) 0x55, dataPackage.getTimestamp(), dataPackage.getData());

        // 读取文件头
        byte[] headerContent = new byte[1024 * 64 * 4 + 24];
        FileUtils.readFile(fileName, headerContent, 0);
        LogFileHeader logFileHeader = new LogFileHeader();
        logFileHeader.setPayload(headerContent);
//        log.debug("读取到log头{}", logFileHeader);

        // 寻找写入位置
        // 读取上一条记录的长度
        int offset = -1;
        if (logFileHeader.getNextWriteIndex() == 0) {
            // 第一条
            offset = headerContent.length + 0;
            log.debug("log文件第一条，offset={}", offset);
        } else if (logFileHeader.getNextWriteIndex() == -1) {
            // 已满
            log.error("log文件最后一条，不应该到这里");
        } else {
            byte[] lastContent = new byte[16];
            FileUtils.readFile(fileName, lastContent, logFileHeader.getIndices()[logFileHeader.getNextWriteIndex() - 1]);
            LogItem lastItem = new LogItem();
            lastItem.setHeader(lastContent);
            offset = lastItem.getLength() + logFileHeader.getIndices()[logFileHeader.getNextWriteIndex() - 1];
            log.debug("log文件非首尾条，offset={}", offset);
        }

        // 写入数据
        FileUtils.appendFile(fileName, logItem.getPayload());

        // 修改文件头
        logFileHeader.getIndices()[logFileHeader.getNextWriteIndex()] = offset;
        logFileHeader.setMinTime(Math.min(logFileHeader.getMinTime(), dataPackage.getTimestamp()));
        logFileHeader.setMaxTime(Math.min(logFileHeader.getMaxTime(), dataPackage.getTimestamp()));
        log.debug("修改文件头索引，将{}位置的索引修改为{}", logFileHeader.getNextWriteIndex(), offset);
        FileUtils.overwriteFile(fileName, logFileHeader.getPayload(), 0);

        return 0;
    }

    public void createNewFile() throws IOException {
        // 创建索引部分
        LogFileHeader header = new LogFileHeader();
        FileUtils.writeFile(fileName, header.getPayload());
    }

    public static void main(String[] args) throws IOException {
        LogFileService service = new LogFileService();

        int version = 0x21030001;
        short operationType = 0x3;
        short responseCode = 200;
        short operationCode = 0x0006;
        short packageCode = 0;
        short packageLength = 0x0054;
        short probeId = 0x0010;
        byte port = 0x01;
        byte dataStrategy = 0;
        byte reserve = 0;
        byte clockSync = (byte) 0xAA;
        short year = 2014;
        short day = 3;
        short hour = 8;
        short minute = 0x2e;
        short second = 0x10;
        int microSecond = 0x000f0b99;
//        byte[] data = {0x00, 0x00, 0x02, 0x55, 0x55, 0x55, 0x00, 0x00, 0x02, 0x44, 0x44, 0x66, 0x08, 0x00, 0x45, 0x00,
//                0x00, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x40, 0x11, 0x3E, (byte) 0x87, 0x03, 0x1B, 0x1B, 0x02, 0x03, 0x1B,
//                0x1B, 0x01, 0x04, 0x00, 0x04, 0x01, 0x00, 0x1A, 0x00, 0x00, (byte) 0xA5, 0x00, 0x66, (byte) 0x80, 0x00, 0x00,
//                0x00, 0x3c, 0x00, 0x01, (byte) 0x99, 0x42, 0x59, (byte) 0xAF, 0x03, (byte) 0xDB, 0x00, 0x23, (byte) 0xc0, (byte) 0x96, 0x0B, 0x78};

        byte[] data = ContentUtils.generatorRandomData(0x00000002555556L, 0x000002444455L, "192.168.1.1",
                "192.168.31.1", (short) 8080, (short) 9090);

        DataPackage dataPackage = new DataPackage(version, operationType, responseCode, 0, operationCode,
                packageCode, probeId, port, dataStrategy, reserve, clockSync, year, day, hour, minute, second,
                microSecond, data);
        System.out.println("dataPackage = " + dataPackage);
        service.createNewFile();
        service.appendLog(dataPackage);
        service.appendLog(dataPackage);
    }


}
